#include "App.h"
#include <Application.h>
#include <GridLayout.h>
#include <GroupLayout.h>
#include <InterfaceDefs.h>
#include <Layout.h>
#include <LayoutItem.h>
#include <Message.h>
#include <Rect.h>
#include <String.h>
#include <StringView.h>
#include <Window.h>
#include <GroupLayoutBuilder.h>
#include <GridLayoutBuilder.h>
#include <Button.h>
#include <cstdio>
#include <format>
#include "Contacts.h"
#include <TextControl.h>

// quick and dirty function to quickly make text labels
BStringView *MakeText(const char* text) 
{
	return new BStringView("rstr", text);
}

App::App(void) : BApplication(std::format("application/x-vnd.{}-{}", APP_AUTHOR, APP_NAME).c_str())
{
	// Initialize BRect frame for window
	BRect frame(100, 100, 500, 400);
	
	// Initialize parent group for all UI elements to sit in. This is so they all get grouped nicely together
	BGroupLayout *pGroup = new BGroupLayout(B_HORIZONTAL);

	// Create window
	BWindow *window = new BWindow(frame, std::format("{}", APP_NAME).c_str(), B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS);

	// Set window layout to parent group created earlier
	window->SetLayout(pGroup);

	pGroup->View()->AdoptSystemColors();

	/*
	// Initialize special UI group for all of our "real" elements
	BGroupLayout *uGroup = new BGroupLayout(B_HORIZONTAL, 100);
	// Add label group to parent group
	pGroup->AddItem(uGroup);

	// Center label group
	uGroup->SetExplicitAlignment(BAlignment(B_ALIGN_HORIZONTAL_CENTER, B_ALIGN_VERTICAL_CENTER));

	// And finally, add a button!!

	uGroup->AddView(connectButton);
	*/
	BGridLayout *uiElems = new BGridLayout(10, 0);
	pGroup->AddItem(uiElems);
	uiElems->SetExplicitAlignment(BAlignment(B_ALIGN_HORIZONTAL_CENTER, B_ALIGN_VERTICAL_CENTER));

	BButton *connectButton = new BButton(frame, "connect", "Connect!", new BMessage(msgConnectButtonClicked));
	connectButton->SetTarget(this);

	BTextControl *jidInput = new BTextControl("jidinput", "JID:", "Placeholder", new BMessage(msgConnectButtonClicked));
	BTextControl *pwInput = new BTextControl("jidinput", "JID:", "Placeholder", new BMessage(msgConnectButtonClicked));

	uiElems->AddView(jidInput, 0, 0);
	uiElems->AddView(MakeText("Password placeholder"), 0, 1);
	uiElems->AddView(connectButton, 0, 2);
	uiElems->SetMinRowHeight(2, 75);

	window->Show();

	App::ConnectWindow = window;
}

void App::MessageReceived(BMessage *msg) 
{
	switch(msg->what) {
		case msgConnectButtonClicked:
			{	
				printf("Connect button clicked...\n");
				App::ConnectWindow->Hide();
				App::ContactsWindow = ShowContacts();
				break;
			}
		default:
			{
				printf("Unknown message received: %d\n", msg->what);
				BApplication::MessageReceived(msg);
				break;
			}
	}
}

int main(void)
{
	App *app = new App();
	app->Run();

	delete app;
	return 0;
}
